#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <db.h>

#include "plog.h"

static int handle_msg(struct provmsg *msg, void *arg)
{
	DB *db = (DB *) arg;
	DBT key, rec;

	key.flags = 0;
	rec.data = (void *) msg;
	rec.size = msg_getlen(msg);
	rec.flags = DB_APPEND;

	return db->put(db, NULL, &key, &rec, DB_APPEND);
}

static void usage(void) {
	printf("Usage: todb <dbfile>\n"
	       "Converts the provenance log on standard input into a Berkeley DB file.\n");
}

int main(int argc, char *argv[])
{
	static plog_handler_t *fns[] = {
		[PROVMSG_BOOT] = handle_msg,
		[PROVMSG_CREDFORK] = handle_msg,
		[PROVMSG_CREDFREE] = handle_msg,
		[PROVMSG_SETID] = handle_msg,
		[PROVMSG_EXEC] = handle_msg,
		[PROVMSG_FILE_P] = handle_msg,
		[PROVMSG_MMAP] = handle_msg,
		[PROVMSG_INODE_P] = handle_msg,
		[PROVMSG_INODE_ALLOC] = handle_msg,
		[PROVMSG_INODE_DEALLOC] = handle_msg,
		[PROVMSG_SETATTR] = handle_msg,
		[PROVMSG_LINK] = handle_msg,
		[PROVMSG_UNLINK] = handle_msg,
		[PROVMSG_MQSEND] = handle_msg,
		[PROVMSG_MQRECV] = handle_msg,
		[PROVMSG_SHMAT] = handle_msg,
		[PROVMSG_SOCKSEND] = handle_msg,
		[PROVMSG_SOCKRECV] = handle_msg,
	};
	int rv;
	DB *db;
	char *dbfile;
	int n;

	if (argc < 2) {
		printf("dbfile argument is required\n\n");
		usage();
		return 1;
	}

	n = strlen(argv[1]);
	dbfile = malloc(n + 3);
	if (!dbfile) {
		fprintf(stderr, "failed to allocate string\n");
		return 1;
	}
	strcpy(dbfile, argv[1]);
	strcat(dbfile, ".db");

	rv = db_create(&db, NULL, 0);
	if (rv) {
		fprintf(stderr, "db_create: %s\n", strerror(rv));
		goto out_free;
	}
	rv = db->open(db, NULL, dbfile, NULL, DB_RECNO,
			DB_CREATE | DB_EXCL, 0644);
	if (rv) {
		db->err(db, rv, NULL);
		goto out_free;
	}

	rv = plog_process(stdin, fns, NUM_PROVMSG_TYPES, (void *) db);
	if (rv)
		db->err(db, rv, NULL);

	db->close(db, 0);
out_free:
	free(dbfile);
	return rv ? 1 : 0;
}
